package stx.libjava.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNotSame;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;

import org.junit.Test;
@SuppressWarnings("rawtypes")
public class ClassLoaderHierarchyTests {

	@Test
	public void testParentOfSystemClassLoader() {
		ClassLoader parent = ClassLoader.getSystemClassLoader().getParent();
//		system->sun.misc.Launcher$AppClassLoader@16f0472
//		parent->sun.misc.Launcher$ExtClassLoader@18d107f
		assertNull(parent.getParent());
	}
	
	@Test
	public void testNiceClassLoaderSelfLoading() throws Exception {
		URL[] urls = new URL[1];
		File file = new File("foos");
		urls[0] = file.toURL();
		URLClassLoader cl = new URLClassLoader(urls);
		Class result = Class.forName("Foo", true, cl);
		assertEquals(cl, result.getClassLoader());
		assertEquals(cl, result.getSuperclass().getClassLoader());
		assertNull(result.getDeclaredField("baz").getClass().getClassLoader());
	}
	
	@Test
	public void testNiceClassLoaderSelfLoadingWithPreCaching() throws Exception {
		URL[] urls = new URL[1];
		File file = new File("foos");
		urls[0] = file.toURL();
		URLClassLoader cl = new URLClassLoader(urls);
		Class result = Class.forName("Foo", true, cl);						
		assertNull(result.getDeclaredField("baz").getClass().getClassLoader());
	}
	
	@Test
	public void testTwoSameClassesInDifferentCLsAreDifferent() throws Exception {
		URL[] urls = new URL[1];
		File file = new File("foos");
		urls[0] = file.toURL();
		URLClassLoader cl1 = new URLClassLoader(urls);
		URLClassLoader cl2 = new URLClassLoader(urls);
		Class result1 = Class.forName("Foo", true, cl1);
		Class result2 = Class.forName("Foo", true, cl2);		
		assertNotSame(result1,result2);		
	}
	
}

