package stx.libjava.tests.mocks;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Nice class loader is following delegation model and is not doing any of the
 * evil tricks mean class loader is doing..
 * 
 * @author m
 * 
 */
public class MeanClassLoader extends ClassLoader {

	private static final int BUFFER_SIZE = 8192;

	protected synchronized Class loadClass(String className, boolean resolve)
			throws ClassNotFoundException {
		
		Class cls = findLoadedClass(className);
		if (cls != null) {
			return cls;
		}
		String clsFile = className.replace('.', '/') + ".class";

		byte[] classBytes = null;
		try {
			InputStream in = getResourceAsStream(clsFile);
			byte[] buffer = new byte[BUFFER_SIZE];
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			int n = -1;
			while ((n = in.read(buffer, 0, BUFFER_SIZE)) != -1) {
				out.write(buffer, 0, n);
			}
			classBytes = out.toByteArray();
		} catch (IOException e) {
			throw new ClassNotFoundException("Cannot load class: " + className);
		}

		if (classBytes == null) {
			throw new ClassNotFoundException("Cannot load class: " + className);
		}

		try {			
			cls = defineClass(className, classBytes, 0, classBytes.length);
			if (resolve) {
				resolveClass(cls);
			}
		} catch (SecurityException e) {
			// loading core java classes such as java.lang.String
			// is prohibited, throws java.lang.SecurityException.
			// delegate to parent if not allowed to load class
			//throw new ClassNotFoundException("Cannot load class: " + className + " because of security restrictions");
			cls = super.loadClass(className, resolve);
		}

		return cls;
	}
	
	
}
